package com.ningdatech.pmapi.dashboard.handle;

import cn.hutool.core.bean.BeanUtil;
import com.google.common.collect.Lists;
import com.ningdatech.pmapi.common.constant.RegionConst;
import com.ningdatech.pmapi.common.helper.RegionCacheHelper;
import com.ningdatech.pmapi.dashboard.model.entity.CockpitStats;
import com.ningdatech.pmapi.dashboard.model.vo.CockpitStatsVO;
import com.ningdatech.pmapi.sys.model.dto.RegionDTO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Classname CockpitStatsHandler
 * @Description
 * @Date 2023/10/9 14:11
 * @Author PoffyZhang
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class CockpitStatsHandler {

    private final RegionCacheHelper regionCacheHelper;

    public CockpitStatsVO convertCockpitStats(CockpitStats cockpitStats) {
        CockpitStatsVO res = BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.class);

        if(Objects.isNull(cockpitStats)){
            return res;
        }
        //监测数据
        res.setMonitorData(convertMonitor(cockpitStats));
        //专家数据
        res.setExpertData(convertExpertData(cockpitStats));
        //顶部数据
        res.setTopData(convertTopData(cockpitStats));
        //地图数据
        res.setMapProjectData(convertMapProjectsData(cockpitStats));
        //项目项目数据
        res.setStatusProjectsData(convertStatusProjectsData(cockpitStats));
        //项目效益 优秀项目
        res.setExcellentData(convertExcellentData(cockpitStats));
        //核减信息
        res.setReduceData(convertReduceData(cockpitStats));
        //产出信息
        res.setProduceData(convertProduceData(cockpitStats));
        //绩效
        res.setPerformanceData(convertPerformanceData(cockpitStats));
        return res;
    }

    private CockpitStatsVO.PerformanceData convertPerformanceData(CockpitStats cockpitStats) {
        CockpitStatsVO.PerformanceData data = BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.PerformanceData.class);
        data.setPerformanceTops(convertTops(cockpitStats));
        return data;
    }

    private List<CockpitStatsVO.PerformanceTop> convertTops(CockpitStats cockpitStats) {
        List<CockpitStatsVO.PerformanceTop> tops = Lists.newArrayList();
        tops.add(new CockpitStatsVO.PerformanceTop("一星",cockpitStats.getPerformanceTop1Score(),cockpitStats.getPerformanceTop1Name()));
        tops.add(new CockpitStatsVO.PerformanceTop("二星",cockpitStats.getPerformanceTop2Score(),cockpitStats.getPerformanceTop2Name()));
        tops.add(new CockpitStatsVO.PerformanceTop("三星",cockpitStats.getPerformanceTop3Score(),cockpitStats.getPerformanceTop3Name()));
        tops.add(new CockpitStatsVO.PerformanceTop("四星",cockpitStats.getPerformanceTop4Score(),cockpitStats.getPerformanceTop4Name()));
        tops.add(new CockpitStatsVO.PerformanceTop("五星",cockpitStats.getPerformanceTop5Score(),cockpitStats.getPerformanceTop5Name()));
        return tops;
    }

    private CockpitStatsVO.ProduceData convertProduceData(CockpitStats cockpitStats) {
        return BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.ProduceData.class);
    }

    private CockpitStatsVO.ReduceData convertReduceData(CockpitStats cockpitStats) {
        CockpitStatsVO.ReduceData data = BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.ReduceData.class);
        data.setAmountComparisonCharts(convertAmountComparisonChart(cockpitStats));
        return data;
    }

    private List<CockpitStatsVO.AmountComparisonChart> convertAmountComparisonChart(CockpitStats cockpitStats) {
        List<CockpitStatsVO.AmountComparisonChart> res = Lists.newArrayList();
        res.add(new CockpitStatsVO.AmountComparisonChart("申报", cockpitStats.getDeclaredAmountApprove(),cockpitStats.getApprovalAmountApprove(),cockpitStats.getDifferenceApprove()));
        res.add(new CockpitStatsVO.AmountComparisonChart("批复", cockpitStats.getDeclaredAmountApproval(),cockpitStats.getApprovalAmountApproval(),cockpitStats.getDifferenceApproval()));
        res.add(new CockpitStatsVO.AmountComparisonChart("合同", cockpitStats.getDeclaredAmountContract(),cockpitStats.getApprovalAmountContract(),cockpitStats.getDifferenceContract()));
        return res;
    }

    private CockpitStatsVO.ExcellentData convertExcellentData(CockpitStats cockpitStats) {
        CockpitStatsVO.ExcellentData data = BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.ExcellentData.class);
        data.setExcellentProjects(Lists.newArrayList());
        return data;
    }

    private CockpitStatsVO.StatusProjectsData convertStatusProjectsData(CockpitStats cockpitStats) {
        return BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.StatusProjectsData.class);
    }

    private List<CockpitStatsVO.MapProjectsData> convertMapProjectsData(CockpitStats cockpitStats) {
        List<CockpitStatsVO.MapProjectsData> mapProjectsDatas = Lists.newArrayList();
        List<RegionDTO> regions = regionCacheHelper.listChildren(RegionConst.RC_LS, RegionConst.RL_COUNTY);
        regions = regions.stream().sorted(Comparator.comparing(RegionDTO::getRegionCode)).collect(Collectors.toList());
        for(RegionDTO region : regions){
            CockpitStatsVO.MapProjectsData mapProjectsData = new CockpitStatsVO.MapProjectsData();
            mapProjectsData.setRegionCode(region.getRegionCode());
            mapProjectsData.setRegionName(region.getRegionName());
            //TODO: 项目数据
            mapProjectsData.setProjectsNum(0);
            mapProjectsDatas.add(mapProjectsData);
        }
        return mapProjectsDatas;
    }

    private CockpitStatsVO.TopData convertTopData(CockpitStats cockpitStats) {
        return BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.TopData.class);
    }

    private CockpitStatsVO.Expert convertExpertData(CockpitStats cockpitStats) {
        CockpitStatsVO.Expert expert = BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.Expert.class);
        expert.setExpertRegionData(buidExpertRegionData(cockpitStats));
        return expert;
    }

    private List<CockpitStatsVO.ExpertRegion> buidExpertRegionData(CockpitStats cockpitStats) {
        List<CockpitStatsVO.ExpertRegion> regionData = Lists.newArrayList();
        List<RegionDTO> regions = regionCacheHelper.listChildren(RegionConst.RC_LS, RegionConst.RL_COUNTY);
        regions = regions.stream().sorted(Comparator.comparing(RegionDTO::getRegionCode)).collect(Collectors.toList());
        for(RegionDTO region : regions){
            CockpitStatsVO.ExpertRegion expertRegion = new CockpitStatsVO.ExpertRegion();
            expertRegion.setRegionCode(region.getRegionCode());
            expertRegion.setRegionName(region.getRegionName());
            //TODO: 区域数据
            expertRegion.setExpertNum(0);
            regionData.add(expertRegion);
        }
        return regionData;
    }

    private CockpitStatsVO.Monitor convertMonitor(CockpitStats cockpitStats) {
        CockpitStatsVO.Monitor monitor = BeanUtil.copyProperties(cockpitStats,CockpitStatsVO.Monitor.class);
        monitor.setPasswordGradeCharts(buidPasswordGradeCharts(cockpitStats));
        monitor.setSecrecyGradeCharts(buidSecrecyGradeCharts(cockpitStats));
        return monitor;
    }

    private List<CockpitStatsVO.SecrecyGradeChart> buidSecrecyGradeCharts(CockpitStats cockpitStats) {
        List<CockpitStatsVO.SecrecyGradeChart> chars = Lists.newArrayList();
        chars.add(new CockpitStatsVO.SecrecyGradeChart(1,cockpitStats.getMonitorSecrecyGrade1Num()));
        chars.add(new CockpitStatsVO.SecrecyGradeChart(2,cockpitStats.getMonitorSecrecyGrade2Num()));
        chars.add(new CockpitStatsVO.SecrecyGradeChart(3,cockpitStats.getMonitorSecrecyGrade3Num()));
        chars.add(new CockpitStatsVO.SecrecyGradeChart(4,cockpitStats.getMonitorSecrecyGrade4Num()));
        chars.add(new CockpitStatsVO.SecrecyGradeChart(5,cockpitStats.getMonitorSecrecyGrade5Num()));
        return chars;
    }

    private List<CockpitStatsVO.PasswordGradeChart> buidPasswordGradeCharts(CockpitStats cockpitStats) {
        List<CockpitStatsVO.PasswordGradeChart> chars = Lists.newArrayList();
        chars.add(new CockpitStatsVO.PasswordGradeChart(1,cockpitStats.getMonitorPasswordGrade1Num()));
        chars.add(new CockpitStatsVO.PasswordGradeChart(2,cockpitStats.getMonitorPasswordGrade2Num()));
        chars.add(new CockpitStatsVO.PasswordGradeChart(3,cockpitStats.getMonitorPasswordGrade3Num()));
        chars.add(new CockpitStatsVO.PasswordGradeChart(4,cockpitStats.getMonitorPasswordGrade4Num()));
        chars.add(new CockpitStatsVO.PasswordGradeChart(5,cockpitStats.getMonitorPasswordGrade5Num()));
        return chars;
    }
}
